unit UDMRainOff;

interface

uses
   SysUtils, Classes, Dialogs, Forms, Controls;

type DataArrayR = array of real;
     DataArrayI = array of integer;
     LineSet = array of string;
     RemoveSet = array of boolean;
type
  TDMRainOff = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
  public
    FileName, OutputFileName,
    Title1, Title2          : string;
    Description             : string[23];
    Code, LinQua, Index     : string[4];
    SerialNr : DataArrayI;
    DataValue, HData, Evap,
    Xdata,Ydata,YrData,
    YcData,Zdata,HlpData,
    Time, Rain, Runoff      : DataArrayR;
    MaxStor, InitStor,
    EscRate, InitRun,
    InitLevel, EffPor,
    Bvalue, Cvalue          : real;
    FileLine                : LineSet;
    ToBeRemoved             : RemoveSet;
    NrOfData, NrOfLines     : integer;
    Analysis, NoDatFile     : boolean;
    Drainage, NoRange       : boolean;
    Quadratic               : boolean;
    procedure ReadInputFile;
    procedure WriteInputFile (const DocName: string);
    procedure ReadOutputLines(const DocName: string);
    procedure WriteOutputLines(const DocName: string);
    procedure ReadOutputIndex;
    procedure ReadOutputData(const GraphType : byte);
    procedure program_error(const Warning: string);
    function  Question(const Query: string): boolean;
  private
    { Private declarations }
  end;

var
  DMRainOff: TDMRainOff;


implementation

uses URainRun, URainOffUtils, URainForm;

{$R *.dfm}


procedure TDMRainOff.DataModuleCreate(Sender: TObject);
var InitialDir: string;
begin
  getdir (0,InitialDir);
  FileName:=InitialDir;
end;


procedure TDMRainOff.ReadInputFile;
var  InFile: textfile;
     i, k: integer;
     SingleChar : char;
     HlpStr:  string;
     HlpStr1: string[8];
     HlpStr2: string[3];
     OutDated, Warn: boolean;

begin
  Assignfile(InFile, FileName);
  {$I-} Reset(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + filename + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  Readln (InFile);
  read (InFile,Code);
  if Code=' Ana' then readln (InFile);
  if Code=' Pre' then
  begin
    readln (Infile,HlpStr1,HlpStr2);
    if HlpStr2='dra' then Drainage:=true
    else Drainage:=false;
  end;
  readln (InFile);
  readln (InFile,Title1);
  readln (InFile,Title2);
  readln (InFile);
  readln(InFile);
  readln (InFile, Description, MaxStor);
  readln (InFile, Description, InitStor);
  readln (InFile, Description, EscRate);
  Warn:=false;
  if Code = ' Ana' then
  begin
    Analysis := true;
    Drainage := false;
    for k:=1 to 4 do readln (InFile);
    k:=0;
    Try while not eof(Infile) do
      begin
        k:=k+1;
        SetLength(SerialNr,k);
        SetLength(Time,k);
        SetLength(Rain,k);
        SetLength(Evap,k);
        SetLength(RunOff,k);
        SetLength(ToBeRemoved,k);
        ToBeRemoved[k-1] := false;
        readln (InFile,SerialNr[k-1],Time[k-1],Rain[k-1],Evap[k-1],HlpStr);
        if Hlpstr='' then
        begin
          RunOff[k-1]:=Evap[k-1];
          Evap[k-1]:=EscRate;
          if not Warn then Showmessage ('Please check the escape rates');
          Warn:=true;
        end else
          RunOff[k-1]:=strtofloat(HlpStr);
        NrOfData:=k;
      end;
      Except on E: exception do
      begin
        Program_error('Real numbers in the inputfile "' + filename +
                      '" are corrupt');
      end;
    end; {Try}
    Halt:=false;
    for i:=1 to k do if Evap[i-1]<0 then
    begin
      if Halt=false then
      Showmessage('There are negative evaporation values, i.e. there is'+
                  ' another source of water. This program cannot handle that.'+
                  ' Use RainOffT instead, unless the prediction mode will'+
                  ' be used.');
      Halt:=true;
//      exit;
    end;
  end else
  begin
    Analysis := false;
    readln (InFile,LinQua);
    for k:=1 to 18 do read(InFile,SingleChar);
    Description := SingleChar;
    readln (InFile, A1coeff);
    for k:=1 to 18 do read(InFile,SingleChar);
    readln (InFile, B1value);
    for k:=1 to 18 do read(InFile,SingleChar);
    OutDated:=false;
    Try read (InFile, A2coeff);
      Except on E: exception do
      begin
        Outdated:=true;
        A2Coeff:=0;
        showmessage('This is an outdated input file. Please make the necessary'+
                    ' adjustments for the Alpha characteristics.');
        for k:=1 to 8 do read(InFile,SingleChar);
        readln (InFile, EffPor);
      end else
        readln(InFile);
    end;
    if not OutDated then
    begin
      readln (InFile);
      for k:=1 to 18 do read(InFile,SingleChar);
      readln (InFile, B2value);
      for k:=1 to 26 do read(InFile,SingleChar);
      readln (InFile, EffPor);
    end;
    for k:=1 to 23 do read(InFile,SingleChar);
    readln (InFile,InitRun);
    for k:=1 to 23 do read(InFile,SingleChar);
    readln (InFile,InitLevel);
    if not OutDated then
    begin
      for k:=1 to 23 do read(InFile,SingleChar);
      readln (InFile,RunRef);
    end;
    for k:=1 to 4 do readln (InFile);
    k:=0;
    Try while not eof(Infile) do
      begin
        k:=k+1;
        SetLength(SerialNr,k);
        SetLength(Time,k);
        SetLength(Rain,k);
        SetLength(Evap,k);
        SetLength(RunOff,k);
        SetLength(ToBeRemoved,k);
        ToBeRemoved[k-1] := false;
        readln (InFile,SerialNr[k-1],Time[k-1],Rain[k-1],HlpStr);
        if Hlpstr='' then
        begin
           Evap[k-1]:=EscRate;
           if not Warn then Showmessage ('Please check the ecape rates');
           Warn:=true;
        end else
          Evap[k-1]:=strtofloat(HlpStr);
      end;
      Except on E: exception do
      begin
        Program_error('Real numbers in the inputfile "' + filename +
                      '" are corrupt');
      end;
    end; {Try}
  end; {if Code='Ana' then else}
  Closefile(InFile);
    with RainForm do
    begin
    if Code='Ana' then
    begin
      AlfaMemo.Visible:=false;
      AlfaButton.Visible:=false;
      TryMemo.Visible:=true;
      StarLabel.Left:=432;
      StarLabel.Top:=180;
      EvapLabel.Left:=281;
    end else
    begin
      AlfaMemo.Visible:=true;
      AlfaButton.Visible:=true;
      TryMemo.Visible:=false;
      StarLabel.Left:=285;
      StarLabel.Top:=175;
      EvapLabel.Left:=270;
    end;
    end;

end; {TDMRainOff.ReadInputFile}



procedure TDMRainOff.WriteInputFile (const DocName: string);
label 1;
var  InFile: textfile;
     k, m : integer;
     Por : real;
begin
1:Assignfile(InFile, FileName);
  {$I-} Rewrite(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + filename + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  writeln (InFile,' Input data for RainOff program');
  if Code=' Ana' then
     writeln (InFile,' Analysis and reconstruction')
  else
  begin
     if DMRainOff.Drainage then writeln (InFile,' Prediction drainage');
     if not DMRainOff.Drainage then writeln (InFile,' Prediction runoff');
  end;     
  writeln (InFile);
  writeln (InFile,Title1);
  writeln (InFile,Title2);
  writeln (InFile);
  writeln (InFile,' Filename: ',Filename);
  if MaxStor>=1 then
     writeln (InFile,' Maximum dead storage: ',MaxStor:8:2)
  else
     writeln (InFile,' Maximum dead storage: ',MaxStor:8);
  if InitStor>=1 then
     writeln (InFile,' Initial dead storage: ',InitStor:8:2)
  else
     writeln (InFile,' Initial dead storage: ',InitStor:8);
  if EscRate>=1 then
     writeln (InFile,' Max.escape/time unit: ',EscRate:8:2)
  else
     writeln (InFile,' Max.escape/time unit: ',EscRate:8);
  if Analysis then
  begin
    writeln (InFile);
    writeln (InFile, ' -------------------------------------------------');
    writeln (InFile, '  Serial    T i m e   R a i n    Esc. rate Run-off');
    writeln (InFile, ' -------------------------------------------------');
  end else
  begin
    writeln (InFile,' Linear reservoir function : Alpha = A * Runoff + B');
    writeln (InFile, ' A1 coefficient : ', format('%10.5e',[A1coeff]));
    writeln (InFile, ' Value of B1 is : ', format('%10.5e',[B1value]));
    writeln (InFile, ' A2 coefficient : ', format('%10.5e',[A2coeff]));
    writeln (InFile, ' Value of B2 is : ', format('%10.5e',[B2value]));
    if EffPor<1 then Por:=100*EffPor
    else Por:=EffPor;
    writeln (InFile, ' Effective porosity (%) : ', Por:7:2);
    if InitRun>=1 then
      writeln (InFile, ' Initial runoff rate : ', InitRun:8:2)
    else
      writeln (InFile, ' Initial runoff rate : ', InitRun:8);
    if InitLevel>=1 then
      writeln (InFile, ' Initial water level : ', InitLevel:8:2)
    else
      writeln (InFile, ' Initial water level : ', InitLevel:8);
      writeln (InFile, ' Runoff division     : ', RunRef:8:2);
    writeln (InFile);
    writeln (InFile, ' ------------------------------------------');
    writeln (InFile, ' Serial    T i m e   Rainfall   Escape rate');
    writeln (InFile, ' ------------------------------------------');
  end;
  m:=0;
  Try for k:=0 to NrOfData-1 do
      if TobeRemoved[k]=false then
      begin
        m:=m+1;
        if Time[k]>=1 then
           write(InFile, m:5,' ',Time[k]:12:2)
        else
           write(InFile, m:5,' ',Time[k]:12);
        if Rain[k]>=1 then
           write(InFile, ' ',Rain[k]:10:2)
        else
           write(InFile, ' ',Rain[k]:10);
        if Evap[k]>=1 then
           write(InFile, ' ',Evap[k]:10:2)
        else
           write(InFile, ' ',Evap[k]:10);
        if Analysis then
        begin
           if RunOff[k]>=1 then
              writeln(InFile, ' ',RunOff[k]:10:2)
           else
              writeln(InFile, ' ',RunOff[k]:10)
        end
        else
           writeln (InFile);
      end;
      Except on E: exception do
           Program_error('Real numbers in the inputfile "' + filename +
                  '" are corrupt');
  end;
  Closefile(InFile);
end; {TDMRainOff.WriteInputFile}



procedure TDMRainOff.ReadOutputLines(const DocName: string);
var  OutFile: textfile;
     k: integer;
begin
  AssignFile(OutFile,DocName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
     Showmessage('The file "' + DocName +
                 '" could not be opened.  It may be damaged,'  +
                 '  not created due to inadequate input data,' +
                 '  in use by another program.');
    exit;
  end;
  k:=0;
  while not eof(OutFile) do
  begin
    k:=k+1;
    SetLength(FileLine,k);
    readln(OutFile,FileLine[k-1]);
  end;
  NrOfLines:=k;
  Close(OutFile);
end; {TRainOff.ReadOutputLines}



procedure TDMRainOff.WriteOutputLines(const DocName: string);
var  OutFile: textfile;
     k: integer;
begin
  AssignFile(OutFile,DocName);
  {$I-} Rewrite(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + DocName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  for k:=1 to NrOfLines do
    writeln(OutFile,FileLine[k-1]);
  Close(OutFile);
end; {TDMRainOff.WriteOutputLines}



procedure TDMRainOff.ReadOutputIndex;
var  OutFile: textfile;
     k : integer;
     Str10 : string[10];
     str4  : string[4];
begin
  AssignFile(OutFile,OutputFileName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + FileName + '"could not be opened.' +
                ' It may be damaged,  not created due to inadequate' +
                'input data, or in use by another program.');
    exit;
  end;
  for k:=1 to 2 do readln (OutFile);
  readln (OutFile,Title1);
  readln (OutFile,Title2);
  for k:=1 to 10 do readln (OutFile,Index);
  readln (OutFile,Index);
  reset (OutFile);
  for k:=1 to 15 do readln (OutFile);
  readln (OutFile,Str10,Str4);
  if Str10='PREDICTION' then
  begin
    Analysis:=false;
    if Str4=' dra' then Drainage:=true
    else Drainage:=false;
  end;
  Close(OutFile);
end; {TDMRainOff.ReadOutputIndex}



procedure TDMRainOff.ReadOutputData (const GraphType : byte);
var OutFile: textfile;
    k, j : integer;
    Detector : string[4];
    Str5   : string[5];
    Str2   : string[2];
    HlpStr : string[40];
    Str10 : string[10];
    Str15 : string[15];
    ExtraText : boolean;
begin
    AssignFile(OutFile,OutputFileName);
    {$I-} Reset(OutFile); {$I+}
    if ioresult <> 0 then
    begin
      Showmessage('The file "' + FileName + '"could not be opened.' +
                  ' It may be damaged,  not created due to inadequate' +
                  'input data, or in use by another program.');
      exit;
    end;
    Detector := '    ';
    if (GraphType=1) or (GraphType=3) or (GraphType=7) or (GraphType=8) then
    begin
       while not (Detector='RECO') do readln (OutFile,Detector);
       for k:=1 to 5 do readln (OutFile);
       NrOfData := 0;
       ExtraText:=false;
       while not eof (OutFile) do
       begin
         Readln (OutFile,HlpStr);
         NrOfData := NrOfData+1;
       end;
       NrOfData := NrOfData-4;
       if HlpStr=  'Coefficient of explanation R square is: ' then
          NrOfData := NrOfData-1;
       if ExtraText then NrOfData:=NrOfData-6;
       reset (OutFile);
       Detector := '';
       while not (Detector='RECO') do readln (OutFile,Detector);
       for k:=1 to 5 do readln (OutFile);
       for k:=0 to NrOfData-1 do
       begin
         SetLength (SerialNr,k+1);
         SetLength (Xdata,k+1);
         SetLength (Ydata,k+1);
         SetLength (Yrdata,k+1);
         SetLength (Zdata,k+1);
         SetLength (Ycdata,k+1);
         readln (OutFile,SerialNr[k],
              Xdata[k],Ydata[k],YrData[k],Zdata[k],YcData[k],YcData[k]);
             {X=time,Y=Rain,Yr=Rech,Z=Runoff meas.,YcData=runoff calc.}
       end;
       if (GraphType=1) or (GraphType=7) then
       begin
         readln (OutFile,hlpstr);
         readln (OutFile,hlpstr);
         Try
           readln (OutFile,HlpStr,Rsq);
           Except on E: exception do
           begin
             Rsq:=0;
             closefile (OutFile);
             exit;
           end;
         end;
       end;
    end;
    if GraphType=7 then
       for k:=0 to NrOfData-1 do
           Xdata[k]:=Zdata[k];
    if (GraphType=2) or (GraphType=4) then
    begin
      while not (Detector='PRED') do readln (OutFile,Detector);
      for k:=1 to 4 do readln (OutFile);
      NrOfData := 0;
      while not eof (OutFile) do
      begin
        Readln (OutFile);
        NrOfData := NrOfData+1;
      end;
      NrOfData := NrOfData-2;
      reset (OutFile);
      Detector := '';
      while not (Detector='PRED') do readln (OutFile,Detector);
      for k:=1 to 4 do readln (OutFile);
      for k:=0 to NrOfData-1 do
      begin
        SetLength (SerialNr,k+1);
        SetLength (Xdata,k+1);
        SetLength (Ydata,k+1);
        SetLength (Yrdata,k+1);
        SetLength (Zdata,k+1);
        SetLength (Ycdata,k+1);
        SetLength (Hdata,k+1);
        readln (OutFile,SerialNr[k],
             Xdata[k],Ycdata[k],YrData[k],Zdata[k],Ydata[k],Hdata[k]);
             {X=time,Yc=Rain,Yr=Rech,Z=Alpha,Y=disch,H=Head}
      end;
    end;
    if (GraphType=5) then
    begin
      K:=0;
      while not (Detector='PRED') do
      begin
        readln (OutFile,Detector);
        K:=K+1;
      end;
      Reset (OutFile);
      for j:=1 to K-1 do readln (OutFile,Detector);
      readln (OutFile, Str10, Detector);
      DMRainOff.Drainage:=false;
      if Detector=' dra' then DMRainOff.Drainage:=true;
      for k:=1 to 4 do readln (OutFile);
      NrOfData := 0;
      while not eof (OutFile) do
      begin
        Readln (OutFile);
        NrOfData := NrOfData+1;
      end;
      NrOfData := NrOfData-2;
      reset (OutFile);
      for K:=1 to 11 do readln (OutFile);
      read (OutFile,Detector);
      read (OutFile,A1coeff);
      read (OutFile,Str5);
      readln (OutFile,B1value);

      readln (OutFile,Detector,A2coeff,Str5,B2value);
      readln (OutFile,Str15,RunRef);
      Detector := '';
      while not (Detector='PRED') do readln (OutFile,Detector);
      for k:=1 to 4 do readln (OutFile);
      for k:=0 to NrOfData-1 do
      begin
        SetLength (SerialNr,k+1);
        SetLength (Xdata,k+1);
        SetLength (Ydata,k+1);
        SetLength (Yrdata,k+1);
        SetLength (Zdata,k+1);
        SetLength (Ycdata,k+1);
        SetLength (Hdata,k+1);
        readln (OutFile,SerialNr[k],
             Xdata[k],Ycdata[k],YrData[k],Zdata[k],Ydata[k],Hdata[k]);
             {X=time,Yc=Rain,Yr=Rech,Z=Alpha,Y=disch,H=head}
      end;
    end;
    if GraphType=6 then
    begin
      while not (Detector='----') do readln (OutFile,Detector);
      for k:=1 to 4 do readln (OutFile);
       NrOfData := 0;
       Detector := '';
       while not (Detector='----') do
       begin
         Readln (OutFile,Detector);
         NrOfData := NrOfData+1;
       end;
       NrOfData := NrOfData-1;
       reset (OutFile);
       for k:=1 to 11 do readln (OutFile);
       readln (OutFile,Str2);
       if Str2='if' then Quadratic:=false else Quadratic:=true;
       Detector := '';
       while not (Detector='----') do readln (OutFile,Detector);
       for k:=1 to 4 do readln (OutFile);
       for k:=0 to NrOfData-1 do
       begin
         SetLength (SerialNr,k+1);
         SetLength (Xdata,k+1);
         SetLength (Ydata,k+1);
         SetLength (Yrdata,k+1);
         SetLength (Ycdata,k+1);
         SetLength (Zdata,k+1);
         readln (OutFile,SerialNr[k],
              Ydata[k],Xdata[k],Yrdata[k],YcData[k]);
             {X=runoff meas.,Yr=Alpha meas., Yc=Alpha calc.}
         Zdata[k]:=Ydata[k];
       end;
     end;

    Close(OutFile);
end; {TDMRainOff.ReadOutputData}


procedure TDMRainOff.program_error(const Warning: string);
begin Showmessage('A program error occurred.' + Warning);
      Application.Terminate;
end; {TDMRainOff.program_error}



function TDMRainOff.Question(const Query: string): boolean;
begin Result := MessageDlg(Query, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end; {TDMEnDrain.Question}


end.
