unit UPrediction;

interface
uses Dialogs, SysUtils, URainForm, UDMRainOff, URainOffUtils;

var
InputFile, OutputFile          : text;
Extens, Codex                  : string[4];
Head, Stor                     : array [0..501] of real;
Rech,   Alpha, TimeP           : array [1..501] of real;
SerNum                         : array [1..501] of integer;
EscMax, StorIni, StorMax       : real;
Inter,  Slope,   Porous        : real;
Avalue, Bcoeff,  Ccoeff        : real;
Method, NumDat,  Phase         : integer;
EE, FF, TT                     : string;
FalseTime, NoRange             : boolean;

Procedure Predict;
procedure Recharge;
procedure Construct;
procedure WriteOut;

implementation


{Main Program}
Procedure Predict;
{****************}

begin
  Recharge;
  if FalseTime then exit;
  if NoRange then exit;
  Construct;
  WriteOut;
end;
{end of Predict}
{**************}


{Recharge is part of Predict}
{---------------------------}
procedure Recharge;

var
AA                                : char;
Defi                              : array [1..501] of real;
DefIni,    Delta,     Ratio       : real;
EscAct,    TimeMax,   TimeMin     : real;
ii,jj,kk                          : integer;

begin
  assign (InputFile,DMRainOff.FileName);
  reset (InputFile);
  for ii:=1 to 3 do readln (InputFile);
  readln (InputFile,DMRainOff.Title1);
  readln (InputFile,DMRainOff.Title2);
  for ii:=1 to 2 do readln (InputFile);
  kk := 23;
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,StorMax);
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,StorIni);
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,EscMax,EE);
  while pos(' ',EE)>0 do delete (EE,Pos(' ',EE),1);
  while pos('.',EE)>0 do delete (EE,Pos('.',EE),1);
  TT:=EE;
  while pos('dm',TT)>0 do delete (TT,Pos('dm',TT),2);
  while pos('m',TT)>0 do delete (TT,Pos('m',TT),1);
  while pos('/',TT)>0 do delete (TT,Pos('/',TT),1);
  while pos('c',TT)>0 do delete (TT,Pos('c',TT),1);
  if EE='mm' then
  begin
    TT:='t';
    EE:='mm/t';
  end;
  if EE='cm' then
  begin
    TT:='t';
    EE:='cm/t';
  end;
  if EE='dm' then
  begin
    TT:='t';
    EE:='dm/t';
  end;
  if EE='m' then
  begin
    TT:='t';
    EE:='m/t';
  end;
  readln (InputFile,Codex);
  if Codex=' Lin' then Method := 1;
  kk := 17;
  if Codex=' Lin' then
  begin
    for jj:=1 to kk do read (InputFile,AA);
    readln (InputFile,A1Coeff);
    for jj:=1 to kk do read (InputFile,AA);
    readln (InputFile,B1value);
    for jj:=1 to kk do read (InputFile,AA);
    readln (InputFile,A2Coeff);
    for jj:=1 to kk do read (InputFile,AA);
    readln (InputFile,B2value);
  end;
  kk := 26;
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,Porous);
  Porous:=Porous/100;
  kk := 23;
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,DMrainoff.InitRun);
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,DMrainoff.InitLevel);
  for jj:=1 to kk do read (InputFile,AA);
  readln (InputFile,RunRef);
  for jj:=1 to 4 do readln (InputFile,Codex);
  if (Codex<>' ---') then Readln (InputFile,Codex);
  jj := 0;
  while not eof(InputFile) do
  begin
    jj := jj+1;
    readln (InputFile,SerNum[jj],TimeP[jj],Rain[jj]);
  end;
  NumDat := jj;
  close (InputFile);

  TimeMax:=TimeP[1];
  TimeMin:=TimeP[1];
  for jj:=2 to NumDat do
  begin
      if TimeP[jj]>TimeP[jj-1] then TimeMax:=TimeP[jj];
      if TimeP[jj]<TimeP[jj-1] then TimeMin:=TimeP[jj];
   end;

    NoRange := false;
    if TimeMax-TimeMin<0.001 then
    begin
      Showmessage ('There is no variation in time ' +
                   'The hydrograph cannot be made');
      exit;
    end;

  DefIni  := StorMax - StorIni;
  Defi[1] := DefIni;
  Rech[1] := Rain[1] - DefIni;
  Stor[1] := StorMax;
  Defi[1] := 0;
  if Rech[1]<0 then 
  begin
    Defi[1] := DefIni - Rain[1];
    Rech[1] := 0;
    Stor[1] := StorIni + Rain[1];
  end;
  FalseTime:=false;
  for jj:=2 to NumDat do
  begin
    Delta    := TimeP[jj]-TimeP[jj-1];
    if Delta<=0 then
    begin
      FalseTime:=true;
      Showmessage ('Incorrect time sequence encountered '+
                   'Please correct the input file');
      exit;
    end;
    Ratio    := Stor[jj-1]/StorMax;
    EscMax   := DMRainOff.Evap[jj];
    EscAct   := EscMax*Delta*Ratio;
    Defi[jj] := Defi[jj-1] + EscAct;
    Rech[jj] := Rain[jj] - Defi[jj];
    Stor[jj] := StorMax;
    Defi[jj] := 0;
    if Rech[jj]<0 then
    begin
      Defi[jj] := Defi[jj-1] + EscAct - Rain[jj];
      Rech[jj] := 0;
      Stor[jj] := Stor[jj-1] - EscAct + Rain[jj];
    end;
  end;
end;
{end of Recharge}
{---------------}


{Construct is part of Predict}
{----------------------------}
procedure Construct;

var
Delta, Factor, Past, Present : real; jj : integer;
RunChange : boolean;

begin
  Run[1]:=DMRainOff.InitRun;
  if (B1Value<0) and (Run[1]<-B1Value) then Run[1]:=1.1*-B1value;
  RunChange:=false;
  if RunRef=0 then
  begin
    RunRef:=-0.001;
    RunChange:=true;
  end;
  Head[1]:=DMRainoff.InitLevel;

  if DMrainoff.Drainage then
  for jj:=2 to NumDat do with DMRainOff do
  begin
    if Run[jj-1]>RunRef then
       Alpha[jj] := A1coeff*Run[jj-1] + B1value
    else
       Alpha[jj] := A2coeff*Run[jj-1] + B2value;
    if (Alpha[jj]<0.001) then Alpha[jj]:=0.01;
    Delta       := TimeP[jj] - TimeP[jj-1];
    Factor      := exp(-Alpha[jj]*Delta);
    Past        := Run[jj-1]*Factor;
    Present     := Rech[jj]*(1-Factor);
    Run[jj]     := Past + Present;
    Head[jj]    := Head[jj-1]*Factor +
                   Rech[jj]*(1-Factor)/(0.8*Porous*Alpha[jj]);
  end;

  if not DMrainoff.Drainage then
  for jj:=2 to NumDat do with DMRainOff do
  begin
    if Run[jj]>RunRef then
       Alpha[jj] := A1coeff*Run[jj-1] + B1value
    else
       Alpha[jj] := A2coeff*Run[jj-1] + B2value;
    if (Alpha[jj]<0.001) then Alpha[jj]:=0.001;
    Delta       := TimeP[jj] - TimeP[jj-1];
    Factor      := exp(-Alpha[jj]*Delta);
    Past        := Run[jj-1]*Factor;
    Present     := Rech[jj]*(1-Factor);
    Run[jj]     := Past + Present;
    Head[jj]    := Head[jj-1] + Rech[jj]-Run[jj];
  end;

  if RunChange then
  RunRef:=0;

end;
{end of Construct}
{----------------}


{WriteOut is part of Precict}
{---------------------------}
procedure WriteOut;

var
jj          : integer;
GG,HH,AA    : string;
Pore        : real;
begin
  DMRainOff.FileName := ChangeFileExt (DMRainOff.FileName,'.out');
  assign (OutputFile,DMRainOff.FileName);
  rewrite (OutputFile);
  writeln (OutputFile,'RESULTS OF RainOff PROGRAM');
  writeln (OutputFile);
  writeln (OutputFile,DMRainOff.Title1);
  writeln (OutputFile,DMRainOff.Title2);
  DMRainOff.FileName := ChangeFileExt (DMRainOff.FileName,'.inp');
  writeln (OutputFile,'Name of input file used: ',DMRainOff.FileName);
  writeln (OutputFile);
  writeln (OutputFile,'Maximum dead storage:   ',StorMax:6:2,' mm');
  writeln (OutputFile,'Initial dead storage:   ',StorIni:6:2,' mm');
  writeln (OutputFile,'Max. escape/time unit:  ',EscMax:6:2,' ',EE);
  Pore:=100*Porous;
  writeln (OutputFile,'Effective porosity (%): ',Pore:6:2);
  if Method=1 then writeln(OutputFile,'linear function: alpha = A * runoff + B');
  if Method=2 then
     writeln(OutputFile,'quadratic function: alpha = A*runoff^2+B*runoff+C');
  if Method=1 then
  begin
      writeln (OutputFile,'A1 =',A1coeff:10:5,' B1 =',B1value:10:5);
      writeln (OutputFile,'A2 =',A2coeff:10:5,' B2 =',B2value:10:5);
      writeln (OutputFile,'RunOff divide =',RunRef:10:5);
  end;
  if Method=2 then
  begin
    writeln (OutputFile,'A =',Avalue:10:5,'   B =',Bcoeff:10:5);
    writeln (OutputFile,'C =',Ccoeff:10:5);
  end;
  writeln (OutputFile);
  if DMRainOff.Drainage then
    writeln (OutputFile,'PREDICTION drainage:')
  else
    writeln (OutputFile,'PREDICTION runoff:');
  writeln (OutputFile,'----------------------------------------------------------------');
  if DMRainOff.Drainage then
  writeln (OutputFile,'Serial    Time   Rain/Irr  Recharge    Alpha     Runoff    Head')
  else
  writeln (OutputFile,'Serial    Time   Rain/Irr  Recharge    Alpha     Runoff   Storage');
  if length(TT)=1 then GG:='      ';
  if length(TT)=2 then GG:='     ';
  if length(TT)=3 then GG:='    ';
  if length(TT)=4 then GG:='   ';
  if length(TT)=5 then GG:='  ';
  if length(TT)=6 then GG:=' ';
  if length(EE)=3 then HH:='       ';
  if length(EE)=4 then HH:='      ';
  if length(EE)=5 then HH:='     ';
  if length(EE)=6 then HH:='    ';
  if length(EE)=7 then HH:='   ';
  if length(EE)=8 then HH:='  ';
  if length(EE)=3 then AA:='                   ';
  if length(EE)=4 then AA:='                  ';
  if length(EE)=5 then AA:='                 ';
  if length(EE)=6 then AA:='                ';
  if length(EE)=7 then AA:='               ';
  if length(EE)=8 then AA:='              ';
  writeln (OutputFile,'        ','   t       mm/t      mm/t       --        mm/t      mm ');
  writeln (OutputFile,'----------------------------------------------------------------');
  for jj:=1 to NumDat do
  begin  
    if SerNum[jj]<10 then write (OutputFile,'   ',SerNum[jj])
    else if SerNum[jj]<100 then write (OutputFile, '  ',SerNum[jj])
    else write (OutputFile,' ',SerNum[jj]);
    write (OutputFile,TimeP[jj]:10:2,Rain[jj]:10:2,Rech[jj]:10:2);
    writeln (OutputFile,Alpha[jj]:10:3,Run[jj]:10:2,Head[jj]:10:2);
  end;
  writeln (OutputFile,'---------------------------------------------------------------');
  close (OutputFile);
end;
{end of WriteOut}
{---------------}

end.
